docs = [
    {
        "path": f"../docs/rag{suffix}",
    }
    for suffix in [
        "",
        "/build/",
        "/build/one.md",
        "/build/hnpagency.md",
        "/build/badkoobeh.md",
        "/query/",
        "/query/one.md",
        "/query/hnpagency.md",
        "/query/badkoobeh.md",
    ]
]
